@@ -5,6 +5,7 @@
 _get_ves()
 {
 	local cmd=$1
+	local rm_empty='sed s/^-[[:space:]]*//'
 	case $cmd in
 		create)
 			# create a new VEID, by increasing the last one
@@ -15,17 +16,17 @@ _get_ves()
 		start|mount|umount|destroy)
 			# stopped VEs
 			/usr/sbin/vzlist -H -S -oveid
-			/usr/sbin/vzlist -H -S -oname | sed s/-//g
+			/usr/sbin/vzlist -H -S -oname | $rm_empty
 			;;
 		stop|enter|exec*)
 			# running VEs
 			/usr/sbin/vzlist -H -oveid
-			/usr/sbin/vzlist -H -oname | sed s/-//g
+			/usr/sbin/vzlist -H -oname | $rm_emtpy
 			;;
 		*)
 			# All VEs
 			/usr/sbin/vzlist -H -a -oveid
-			/usr/sbin/vzlist -H -a -oname | sed s/-//g
+			/usr/sbin/vzlist -H -a -oname | $rm_empty
 			;;
 	esac
 }
@@ -57,9 +58,10 @@ _vzctl()
 		--dgramrcvbuf --oomguarpages --lockedpages --privvmpages \
 		--shmpages --numfile --numflock --numpty --numsiginfo \
 		--dcachesize --numiptent --physpages --cpuunits --cpulimit \
-		--iptables --netdev_add --netdev_del --veth_add --veth_del \
+		--iptables --netdev_add --netdev_del --netif_add --netif_del \
 		--diskspace --diskinodes --quotatime --quotaugidlimit \
-		--noatime --capability --devnodes --applyconfig --name"
+		--noatime --capability --devnodes --devices --applyconfig \
+		--name --ioprio --setmode --meminfo"
 
 	iptables_names="iptable_filter iptable_mangle ipt_limit
 		ipt_multiport ipt_tos ipt_TOS ipt_REJECT ipt_TCPMSS
@@ -116,6 +118,9 @@ _vzctl()
 			--onboot|--disabled|--noatime)
 				COMPREPLY=( $( compgen -W "yes no" -- $cur ) )
 				;;
+			--setmode)
+				COMPREPLY=( $( compgen -W "restart ignore" -- $cur ) )
+				;;
 			--config|--applyconfig)
 				local configs=$(ls -1 /etc/vz/conf/*.conf-sample | \
 						    cut -d "-" -f 2- | sed -e 's#.conf-sample$##')
@@ -132,6 +137,15 @@ _vzctl()
 
 				COMPREPLY=( $( compgen -W "$devs" -- $cur ) )
 				;;
+			--meminfo)
+				local meminfo="none privvmpages:1"
+				COMPREPLY=( $( compgen -W "$meminfo" -- $cur ) )
+				;;
+			--ioprio)
+				# ioprio range: 0 - 7 (default : 4)
+				local ioprio='0 1 2 3 4 5 6 7'
+				COMPREPLY=( $( compgen -W "$ioprio" -- $cur ) )
+				;;
 			--capability)
 				# capname:on|off
 				COMPREPLY=( $( compgen -W "$cap_names" -- $cur ) )
@@ -152,6 +166,11 @@ _vzctl()
 				local devs=''
 				COMPREPLY=( $( compgen -W "$devs" -- $cur ) )
 				;;
+			--devices)
+				# FIXME: b|c:major:minor|all:r|w|rw
+				local devices=''
+				COMPREPLY=( $( compgen -W "$devices" -- $cur ) )
+				;;
 			--ipdel)
 				# Get VEID
 				local ve=${COMP_WORDS[2]}
@@ -164,6 +183,22 @@ _vzctl()
 				local ips="`/usr/sbin/vzlist -H -o ip $LIST_OPT $ve | grep -vi -`"
 				COMPREPLY=( $( compgen -W "$ips all" -- $cur ) )
 				;;
+			--netif_del)
+				local netif
+				if [ -f /proc/vz/veth ]; then
+					local ve=${COMP_WORDS[2]}
+					if [[ ! ${ve} != '^[1-9][0-9]*$' ]] ; then
+						# --verbose or --quiet used
+						ve=${COMP_WORDS[3]}
+					fi
+					# get veth(ernet) device of VE, proc and config
+					local netif_proc="`awk "/${ve}/ { print \\$4 }" /proc/vz/veth`"
+					local netif_conf="`grep ^NETIF /etc/vz/conf/${ve}.conf | \
+						sed -e 's/"/\n/g' -e 's/;/\n/g' -e 's/,/\n/g' -e 's/=/ /g' | awk '/^ifname/ {print \$2}'`"
+					netif="$netif_proc $netif_conf"
+				fi
+				COMPREPLY=( $( compgen -W "$netif" -- $cur ) )
+				;;
 			--private|--root)
 				# FIXME
 				# Dir autocompletion works bad since there is
